@@ -5,7 +5,6 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-
 #include "FlatpakResource.h"
 #include "FlatpakBackend.h"
 #include "FlatpakSourcesBackend.h"
@@ -20,18 +19,18 @@
 #include <KFormat>
 #include <KLocalizedString>
 
-#include <QProcess>
-#include <QDir>
+#include <AppStreamQt/release.h>
 #include <QDebug>
 #include <QDesktopServices>
-#include <QIcon>
+#include <QDir>
 #include <QFileInfo>
+#include <QIcon>
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
 #include <QNetworkRequest>
+#include <QProcess>
 #include <QStringList>
 #include <QTimer>
-#include <AppStreamQt/release.h>
 
 static QString iconCachePath(const AppStream::Icon &icon)
 {
@@ -39,21 +38,22 @@ static QString iconCachePath(const AppStream::Icon &icon)
     return QStringLiteral("%1/icons/%2").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation), icon.url().fileName());
 }
 
-FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakInstallation* installation, FlatpakBackend *parent)
+FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakInstallation *installation, FlatpakBackend *parent)
     : AbstractResource(parent)
     , m_appdata(component)
-    , m_id({installation, QString(), FlatpakResource::DesktopApp, component.id(), QString(), QString() })
+    , m_id({FlatpakResource::DesktopApp, component.id(), QString(), QString()})
     , m_downloadSize(0)
     , m_installedSize(0)
-    , m_propertyStates({{DownloadSize, NotKnownYet}, {InstalledSize, NotKnownYet},{RequiredRuntime, NotKnownYet}})
+    , m_propertyStates({{DownloadSize, NotKnownYet}, {InstalledSize, NotKnownYet}, {RequiredRuntime, NotKnownYet}})
     , m_state(AbstractResource::None)
+    , m_installation(installation)
 {
     setObjectName(packageName());
 
     // Start fetching remote icons during initialization
     const auto icons = m_appdata.icons();
     if (!icons.isEmpty()) {
-        foreach (const AppStream::Icon &icon, icons) {
+        for (const AppStream::Icon &icon : icons) {
             if (icon.kind() == AppStream::Icon::KindRemote) {
                 const QString fileName = iconCachePath(icon);
                 if (!QFileInfo::exists(fileName)) {
@@ -61,7 +61,7 @@ FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakI
                     // Create $HOME/.cache/discover/icons folder
                     cacheDir.mkdir(QStringLiteral("icons"));
                     QNetworkAccessManager *manager = new QNetworkAccessManager(this);
-                    connect(manager, &QNetworkAccessManager::finished, this, [this, icon, fileName, manager] (QNetworkReply *reply) {
+                    connect(manager, &QNetworkAccessManager::finished, this, [this, icon, fileName, manager](QNetworkReply *reply) {
                         if (reply->error() == QNetworkReply::NoError) {
                             QByteArray iconData = reply->readAll();
                             QFile file(fileName);
@@ -125,18 +125,28 @@ bool FlatpakResource::canExecute() const
     return (m_id.type == DesktopApp && (m_state == AbstractResource::Installed || m_state == AbstractResource::Upgradeable));
 }
 
-void FlatpakResource::updateFromRef(FlatpakRef* ref)
+void FlatpakResource::updateFromRef(FlatpakRef *ref)
 {
     setArch(QString::fromUtf8(flatpak_ref_get_arch(ref)));
     setBranch(QString::fromUtf8(flatpak_ref_get_branch(ref)));
     setCommit(QString::fromUtf8(flatpak_ref_get_commit(ref)));
     setFlatpakName(QString::fromUtf8(flatpak_ref_get_name(ref)));
-    setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? DesktopApp :
-                                          extends().isEmpty() ? Runtime
-                                                              : Extension);
+    setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? DesktopApp : extends().isEmpty() ? Runtime : Extension);
     setObjectName(packageName());
 }
 
+void FlatpakResource::updateFromAppStream()
+{
+    const QString refstr = m_appdata.bundle(AppStream::Bundle::KindFlatpak).id();
+    g_autoptr(GError) localError = nullptr;
+    g_autoptr(FlatpakRef) ref = flatpak_ref_parse(refstr.toUtf8().constData(), &localError);
+    if (!ref) {
+        qDebug() << "failed to obtain ref" << refstr << localError->message;
+        return;
+    }
+    updateFromRef(ref);
+}
+
 QString FlatpakResource::ref() const
 {
     return typeAsString() + QLatin1Char('/') + flatpakName() + QLatin1Char('/') + arch() + QLatin1Char('/') + branch();
@@ -179,20 +189,13 @@ QVariant FlatpakResource::icon() const
         ret = QIcon(m_bundledIcon);
     } else if (icons.isEmpty()) {
         ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
-    } else foreach(const AppStream::Icon &icon, icons) {
-        switch (icon.kind()) {
+    } else
+        for (const AppStream::Icon &icon : icons) {
+            switch (icon.kind()) {
             case AppStream::Icon::KindLocal:
-            case AppStream::Icon::KindCached: {
-                const QString path = m_iconPath + icon.url().path();
-                if (QFileInfo::exists(path)) {
-                    ret.addFile(path, icon.size());
-                } else {
-                    const QString altPath = m_iconPath + QStringLiteral("%1x%2/").arg(icon.size().width()).arg(icon.size().height()) + icon.url().path();
-                    if (QFileInfo::exists(altPath)) {
-                        ret.addFile(altPath, icon.size());
-                    }
-                }
-            }   break;
+            case AppStream::Icon::KindCached:
+                ret.addFile(icon.url().toLocalFile(), icon.size());
+                break;
             case AppStream::Icon::KindStock: {
                 const auto ret = QIcon::fromTheme(icon.name());
                 if (!ret.isNull())
@@ -208,8 +211,8 @@ QVariant FlatpakResource::icon() const
             }
             case AppStream::Icon::KindUnknown:
                 break;
+            }
         }
-    }
 
     if (ret.isNull()) {
         ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
@@ -237,12 +240,12 @@ int FlatpakResource::installedSize() const
 AbstractResource::Type FlatpakResource::type() const
 {
     switch (m_id.type) {
-        case FlatpakResource::Runtime:
-            return Technical;
-        case FlatpakResource::Extension:
-            return Addon;
-        default:
-            return Application;
+    case FlatpakResource::Runtime:
+        return Technical;
+    case FlatpakResource::Extension:
+        return Addon;
+    default:
+        return Application;
     }
 }
 
@@ -266,7 +269,7 @@ QUrl FlatpakResource::donationURL()
     return m_appdata.url(AppStream::Component::UrlKindDonation);
 }
 
-QString FlatpakResource::flatpakFileType() const
+FlatpakResource::FlatpakFileType FlatpakResource::flatpakFileType() const
 {
     return m_flatpakFileType;
 }
@@ -308,7 +311,7 @@ QString FlatpakResource::name() const
 
 QString FlatpakResource::origin() const
 {
-    return m_id.origin;
+    return m_origin;
 }
 
 QString FlatpakResource::packageName() const
@@ -349,8 +352,9 @@ QString FlatpakResource::sizeDescription()
 {
     KFormat f;
     if (!isInstalled() || canUpgrade()) {
-        if (propertyState(DownloadSize) == NotKnownYet || propertyState(InstalledSize) == NotKnownYet || propertyState(DownloadSize) == Fetching || propertyState(InstalledSize) == Fetching) {
-            qobject_cast<FlatpakBackend*>(backend())->updateAppSize(this);
+        if (propertyState(DownloadSize) == NotKnownYet || propertyState(InstalledSize) == NotKnownYet || propertyState(DownloadSize) == Fetching
+            || propertyState(InstalledSize) == Fetching) {
+            qobject_cast<FlatpakBackend *>(backend())->updateAppSize(this);
             return i18n("Retrieving size information");
         } else if (propertyState(DownloadSize) == UnknownOrFailed || propertyState(InstalledSize) == UnknownOrFailed) {
             return i18n("Unknown size");
@@ -381,13 +385,13 @@ FlatpakResource::ResourceType FlatpakResource::resourceType() const
 QString FlatpakResource::typeAsString() const
 {
     switch (m_id.type) {
-        case FlatpakResource::Runtime:
-        case FlatpakResource::Extension:
-            return QLatin1String("runtime");
-        case FlatpakResource::DesktopApp:
-        case FlatpakResource::Source:
-        default:
-            return QLatin1String("app");
+    case FlatpakResource::Runtime:
+    case FlatpakResource::Extension:
+        return QLatin1String("runtime");
+    case FlatpakResource::DesktopApp:
+    case FlatpakResource::Source:
+    default:
+        return QLatin1String("app");
     }
 }
 
@@ -408,7 +412,7 @@ void FlatpakResource::invokeApplication() const
     g_autoptr(GCancellable) cancellable = g_cancellable_new();
     g_autoptr(GError) localError = nullptr;
 
-    if (!flatpak_installation_launch(m_id.installation,
+    if (!flatpak_installation_launch(m_installation,
                                      flatpakName().toUtf8().constData(),
                                      arch().toUtf8().constData(),
                                      branch().toUtf8().constData(),
@@ -421,7 +425,7 @@ void FlatpakResource::invokeApplication() const
 
 void FlatpakResource::fetchChangelog()
 {
-    emit changelogFetched(AppStreamUtils::changelogToHtml(m_appdata));
+    Q_EMIT changelogFetched(AppStreamUtils::changelogToHtml(m_appdata));
 }
 
 void FlatpakResource::fetchScreenshots()
@@ -459,7 +463,7 @@ void FlatpakResource::setDownloadSize(int size)
     Q_EMIT sizeChanged();
 }
 
-void FlatpakResource::setFlatpakFileType(const QString &fileType)
+void FlatpakResource::setFlatpakFileType(FlatpakFileType fileType)
 {
     m_flatpakFileType = fileType;
 }
@@ -485,12 +489,12 @@ void FlatpakResource::setInstalledSize(int size)
 
 void FlatpakResource::setOrigin(const QString &origin)
 {
-    m_id.origin = origin;
+    m_origin = origin;
 }
 
 void FlatpakResource::setPropertyState(FlatpakResource::PropertyKind kind, FlatpakResource::PropertyState newState)
 {
-    auto & state = m_propertyStates[kind];
+    auto &state = m_propertyStates[kind];
     if (state != newState) {
         state = newState;
 
@@ -527,10 +531,10 @@ void FlatpakResource::setType(FlatpakResource::ResourceType type)
 
 QString FlatpakResource::installationPath() const
 {
-    return installationPath(m_id.installation);
+    return installationPath(m_installation);
 }
 
-QString FlatpakResource::installationPath(FlatpakInstallation* flatpakInstallation)
+QString FlatpakResource::installationPath(FlatpakInstallation *flatpakInstallation)
 {
     g_autoptr(GFile) path = flatpak_installation_get_path(flatpakInstallation);
     g_autofree char *path_str = g_file_get_path(path);
@@ -559,7 +563,7 @@ QDate FlatpakResource::releaseDate() const
 
 QString FlatpakResource::sourceIcon() const
 {
-    const auto sourceItem = qobject_cast<FlatpakBackend*>(backend())->sources()->sourceById(origin());
+    const auto sourceItem = qobject_cast<FlatpakBackend *>(backend())->sources()->sourceById(origin());
     if (!sourceItem) {
         qWarning() << "Could not find source " << origin();
         return QStringLiteral("flatpak-discover");
@@ -571,7 +575,7 @@ QString FlatpakResource::sourceIcon() const
     return iconUrl;
 }
 
-    QString FlatpakResource::author() const
+QString FlatpakResource::author() const
 {
     return m_appdata.developerName();
 }
@@ -583,7 +587,7 @@ QStringList FlatpakResource::extends() const
 
 QSet<QString> FlatpakResource::alternativeAppstreamIds() const
 {
-    const AppStream::Provided::Kind AppStream_Provided_KindId = (AppStream::Provided::Kind) 12; //Should be AppStream::Provided::KindId when released
+    const AppStream::Provided::Kind AppStream_Provided_KindId = (AppStream::Provided::Kind)12; // Should be AppStream::Provided::KindId when released
     const auto ret = m_appdata.provided(AppStream_Provided_KindId).items();
 
     return QSet<QString>(ret.begin(), ret.end());
@@ -593,3 +597,19 @@ QStringList FlatpakResource::mimetypes() const
 {
     return m_appdata.provided(AppStream::Provided::KindMimetype).items();
 }
+
+QString FlatpakResource::versionString()
+{
+    QString version;
+    if (isInstalled()) {
+        auto ref = qobject_cast<FlatpakBackend *>(backend())->getInstalledRefForApp(this);
+        version = flatpak_installed_ref_get_appdata_version(ref);
+    } else if (!m_appdata.releases().isEmpty()) {
+        auto release = m_appdata.releases().constFirst();
+        version = release.version();
+    } else {
+        version = m_id.branch;
+    }
+
+    return AppStreamUtils::versionString(version, m_appdata);
+}
