@@ -47,9 +47,14 @@ public:
     };
     Q_ENUM(ResourceType)
 
+    enum FlatpakFileType {
+        NotAFile,
+        FileFlatpak,
+        FileFlatpakRef,
+    };
+    Q_ENUM(FlatpakFileType)
+
     struct Id {
-        FlatpakInstallation *const installation;
-        QString origin;
         FlatpakResource::ResourceType type;
         const QString id;
         QString branch;
@@ -61,9 +66,7 @@ public:
         bool operator==(const Id &other) const
         {
             return &other == this
-                || (other.installation == installation //
-                    && other.origin == origin //
-                    && other.type == type //
+                || (other.type == type //
                     && other.id == id //
                     && other.branch == branch //
                     && other.arch == arch //
@@ -101,7 +104,7 @@ public:
     QUrl helpURL() override;
     QUrl bugURL() override;
     QUrl donationURL() override;
-    QString flatpakFileType() const;
+    FlatpakFileType flatpakFileType() const;
     QString flatpakName() const;
     QJsonArray licenses() override;
     QString longDescription() override;
@@ -122,10 +125,11 @@ public:
     QDate releaseDate() const override;
     QString author() const override;
     QStringList extends() const override;
+    QString versionString() override;
 
     FlatpakInstallation *installation() const
     {
-        return m_id.installation;
+        return m_installation;
     }
 
     void invokeApplication() const override;
@@ -139,7 +143,7 @@ public:
     void setDownloadSize(int size);
     void setIconPath(const QString &path);
     void setInstalledSize(int size);
-    void setFlatpakFileType(const QString &fileType);
+    void setFlatpakFileType(FlatpakFileType fileType);
     void setFlatpakName(const QString &name);
     void setOrigin(const QString &origin);
     void setPropertyState(PropertyKind kind, PropertyState state);
@@ -154,6 +158,7 @@ public:
     QString ref() const;
     QString sourceIcon() const override;
     QString installPath() const;
+    void updateFromAppStream();
 
 Q_SIGNALS:
     void propertyStateChanged(FlatpakResource::PropertyKind kind, FlatpakResource::PropertyState state);
@@ -168,7 +173,7 @@ private:
     QPixmap m_bundledIcon;
     QString m_commit;
     int m_downloadSize;
-    QString m_flatpakFileType;
+    FlatpakFileType m_flatpakFileType = FlatpakResource::NotAFile;
     QString m_flatpakName;
     QString m_iconPath;
     int m_installedSize;
@@ -176,11 +181,13 @@ private:
     QUrl m_resourceFile;
     QString m_runtime;
     AbstractResource::State m_state;
+    FlatpakInstallation *const m_installation;
+    QString m_origin;
 };
 
 inline uint qHash(const FlatpakResource::Id &key)
 {
-    return qHash(key.installation) ^ qHash(key.origin) ^ qHash(key.type) ^ qHash(key.id) ^ qHash(key.branch) ^ qHash(key.arch);
+    return qHash(key.type) ^ qHash(key.id) ^ qHash(key.branch) ^ qHash(key.arch);
 }
 
 #endif // FLATPAKRESOURCE_H
